import json
import requests

HEADERS = {
    # https://www.whatismybrowser.com/guides/the-latest-user-agent/firefox
    "User-Agent": "Mozilla/5.0 (X11; Linux x86_64; rv:144.0) Gecko/20100101 Firefox/144.0",
    "x-locale": "pl_PL",
    "Host": "www.filmweb.pl",
    "Referer": "https://www.filmweb.pl",
    "Accept": "*/*",
    "Accept-Language": "en-US,en;q=0.5",
    "Accept-Encoding": "gzip, deflate, br, zstd",
    "Origin": "https://www.filmweb.pl",
    "DNT": "1",
    "Connection": "keep-alive",
    "Upgrade-Insecure-Requests": "1",
    "Sec-Fetch-Dest": 'empty',
    "Sec-Fetch-Mode": "cors",
    "Sec-Fetch-Site": "same-origin",
    "Sec-GPC": "1",
    "TE": "trailers",
}

def get_films_page(args):
    """
    request films page
    """
    # this workaround is necessary because multiprocessing imap takes one arg only
    (cookie, user, friend_query, n) = args
    if friend_query:
        url = f"https://www.filmweb.pl/api/v1/logged/friend/{user}/vote/title/film?page={n}"
    else:
        url = f"https://www.filmweb.pl/api/v1/logged/vote/title/film?page={n}"
    data = _get_json(url, cookie, "get_films_page")
    return json.dumps(data)

def auth_check(cookie):
    """
    Check if auth is OK (valid cookie after login)
    """
    url = "https://www.filmweb.pl/api/v1/logged/info"
    content = _get_json(url, cookie, "auth_check")
    user = content["name"]
    return user

def get_votes_count(user):
    """
    Get total count of voteshttps://www.filmweb.pl/api/v1/user/{user}/votes/film/count
    Args:
        user: user to get ratings for
    """
    url = f"https://www.filmweb.pl/api/v1/user/{user}/votes/film/count"
    return _get_json(url, "", "get_votes_count")

def get_global_info(movie_id):
    """
    Get info about a movie (title etc)
    """
    url = f"https://www.filmweb.pl/api/v1/title/{movie_id}/info"
    data = _get_json(url, "", "get_global_info")
    data["entity"] = movie_id
    return json.dumps(data)

def get_global_rating(movie_id):
    """
    Get global rating for a movie
    """
    url = f"https://www.filmweb.pl/api/v1/film/{movie_id}/rating"
    data = _get_json(url, "", "get_global_rating")
    data["entity"] = movie_id
    data["global_rate"] = data.pop("rate")
    return json.dumps(data)

def _get_json(url, cookie, func_name):
    """
    Wrapper for request and unified error
    """
    try:
        response = requests.get(url, headers={"Cookie": cookie, **HEADERS})
        response.raise_for_status()
        content = response.json()
    except Exception as e:
        raise ValueError(f"Failure in {func_name}: {str(e)}")
    else:
        return content


"""filmweb

Usage:
    filmweb [--format=<fileformat>]... [--debug] <username> <cookie>

Options:
    -h --help                     Show this screen
    -f --format=<fileformat>      Output file format: json (default), csv, letterboxd
    -d --debug                    Debug prints
"""

from docopt import docopt
import itertools
import json
import re
import logging
from math import ceil
import multiprocessing
import tqdm
from . import getter
from . import parser

PARALLEL_PROC = multiprocessing.cpu_count()
MOVIES_PER_PAGE = 100
FORMATS = {"csv", "json", "letterboxd"}

def main():
    args = docopt(__doc__)
    user = args["<username>"]
    cookie = args["<cookie>"]
    assert all([user, cookie]), "Empty arguments provided"
    formats = set(f.lower() for f in (args["--format"] or ("json", )))
    assert (formats and formats.issubset(FORMATS)), f"Supported file formats: {FORMATS}"
    if args["--debug"]:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.INFO)
    pool = multiprocessing.Pool(processes=PARALLEL_PROC)
    try:
        logging.info("Checking args...")
        cookie = re.sub("Cookie:", "", cookie).strip()
        votes_total = getter.get_votes_count(user)
        pages = ceil(votes_total/MOVIES_PER_PAGE)
        logged_in_user = getter.auth_check(cookie)
        friend_query = (user != logged_in_user)
        logging.info("Fetching list of movies [1/4]...")
        get_films_page_args = ((cookie, user, friend_query, page) for page in range(1, pages+1))
        user_ratings_raw = tuple(tqdm.tqdm(pool.imap_unordered(getter.get_films_page, get_films_page_args), total=pages))
        user_ratings = tuple(itertools.chain.from_iterable((json.loads(el) for el in user_ratings_raw)))
        total_movies = len(user_ratings)
        logging.info(f"User {user} has {total_movies} movies...")
        assert total_movies, "No movies available"
        ids = tuple(el.get("entity") for el in user_ratings)
        logging.info("Fetching info about movies [2/4]...")
        global_info = tuple(json.loads(el) for el in tqdm.tqdm(pool.imap_unordered(getter.get_global_info, ids), total=total_movies))
        logging.info("Fetching global rating for movies [3/4]...")
        global_rating = tuple(json.loads(el) for el in tqdm.tqdm(pool.imap_unordered(getter.get_global_rating, ids), total=total_movies))
        logging.info("Writing data [4/4]...")
        movies = parser.merge_data(ids, user_ratings, global_info, global_rating)
        parser.write_data(movies, user, formats)
    except Exception as e:
        logging.error(f"Program error: {str(e)}")
    finally:
        pool.close()

if __name__ == "__main__":
    main()



import csv
import logging
import json
from datetime import datetime
from urllib.parse import quote_plus

KEY_MAPPING = {
    "timestamp": "timestamp",
    "favorite": "favorite",
    "rate": "user_rating",
    "global_rate": "global_rating",
    "count": "global_rating_count",
    "originalTitle": "original_title",
    "title": "pl_title",
    "year": "year",
    "entity": "movie_id",
    "url": "url",
    "date": "date",
}
# TODO? country/genre info not visible in api, would need to parse htmls
# TODO? countVote1 and so on keys ingnored for now, histogram info?

def merge_data(ids, user_ratings, global_info, global_rating):
    """
    Merge all data into one
    """
    all_data = tuple(_movie_id_key(el) for el in (user_ratings, global_info, global_rating))
    merged = ({**all_data[0][id], **all_data[1][id], **all_data[2][id]} for id in ids)
    return tuple(_fix_keys(entry) for entry in merged)

def _movie_id_key(data):
    """
    Parse and reformat data into dict with movie_id as key
    """
    return {entry["entity"]: entry for entry in data}

def _fix_keys(entry):
    """
    Fix keys names for data
    """
    fixed = {new_key: entry.get(old_key) for old_key, new_key in KEY_MAPPING.items()}
    if fixed.get("original_title") is None:
        fixed["original_title"] = fixed["pl_title"]
    path = quote_plus(f"""{fixed["pl_title"].strip()}-{fixed["year"]}-{fixed["movie_id"]}""")
    fixed["url"] = f"https://www.filmweb.pl/film/{path}"
    fixed["date"] = datetime.fromtimestamp(fixed["timestamp"]/1000).strftime("%Y-%m-%d")
    return fixed

def _write_csv(file_name, field_names, movies):
    """
    Helper for writing csv
    """
    with open(file_name, "w", encoding="utf-8") as out_file:
        writer = csv.DictWriter(out_file, fieldnames=field_names, dialect="unix")
        writer.writeheader()
        for movie in movies:
            writer.writerow(movie)
    return True

def _letterbox_entry(entry):
    """
    Format letterboxd column names
    https://letterboxd.com/about/importing-data/
    """
    fixed = {
        "Title": entry.get("original_title"),
        "Year": entry.get("year"),
        "Rating10": entry.get("user_rating"),
        "WatchedDate": entry.get("date"),
    }
    return fixed

def write_data(movies, user, formats):
    """
    """
    assert movies, "no data to write"
    date = datetime.now().strftime("%Y%m%d")
    if "json" in formats:
        file_name = f"{user}_{date}.json"
        with open(file_name, "w", encoding="utf-8") as out_file:
            out_file.write(json.dumps(movies))
        logging.info(f"{file_name} written!")
    if "csv" in formats:
        file_name = f"{user}_{date}.csv"
        _write_csv(file_name, KEY_MAPPING.values(), movies)
        logging.info(f"{file_name} written!")
    if "letterboxd" in formats:
        file_name = f"{user}_{date}_letterboxd.csv"
        movies_letterbox = tuple(_letterbox_entry(entry) for entry in movies)
        _write_csv(file_name, movies_letterbox[0].keys(), movies_letterbox)
        logging.info(f"{file_name} written!")
    return file_name
